/*
 * Copyright 2014 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.rmi;

/**
 * Interface of the business object for projects.
 * According to the SQL table, we define some getter an setter here.
 * 
 * <pre>
 * CREATE TABLE aktien (
 *   id NUMERIC default UNIQUEKEY('wertpapier'),
 *   name varchar(255) NOT NULL,
 *   symbol varchar(20) NOT NULL,
 *   lastTradeDate date,
 *   lastTradeTime time,
 *   lastTradePriceOnly float,
 *   previousClose float,
 *   currency varchar(5),
 *   value float,
 *   yearLow float,
 *   yearHigh float,
 *   change float,
 *   changeInPercentage float,
 *   type varchar(20).
 *   exchange varchar(50),
 *   comment text, 
 *   UNIQUE (id),
 *   PRIMARY KEY (id)
 * );
 * </pre>
 * 
 * <br>
 * Getters and setters for the primary key (id) are not needed.
 * Every one of the following methods has to throw a RemoteException. <br>
 */

import java.rmi.RemoteException;
import java.sql.Time;
import java.util.Date;

import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierInfoBean;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBObject;

/**
 * @author munstfr
 * 
 */
public interface Wertpapier extends DBObject {

	/**
	 * Liefert die Beschreibung der Aktien.
	 * 
	 * @return die Beschreibung der Aktien.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	String getComment() throws RemoteException;

	/**
	 * Liefert den lastTradeDate der Wertpapier.
	 * 
	 * @return lastTradeDate der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 Date getLastTradeDate() throws RemoteException;

	/**
	 * Liefert den lastTradePriceOnly der Wertpapier.
	 * 
	 * @return lastTradePriceOnly der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 double getLastTradePriceOnly() throws RemoteException;

	/**
	 * Liefert den lastTradeTime der Wertpapier.
	 * 
	 * @return lastTradeTime der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 Time getLastTradeTime() throws RemoteException;

	/**
	 * Liefert den Name der Wertpapier.
	 * 
	 * @return name Der Name der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 String getName() throws RemoteException;

	/**
	 * Liefert eine Liste mit alle Orders dieser Wertpapier
	 * 
	 * @return Liste mit OrderImpl der Wertpapier
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 DBIterator<?> getOrderHistory() throws RemoteException;

	/**
	 * Liefert den previousClose der Wertpapier.
	 * 
	 * @return previousClose der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 double getPreviousClose() throws RemoteException;

	/**
	 * Liefert den Name des Symbols.
	 * 
	 * @return symbol Das Symbol der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 String getSymbol() throws RemoteException;

	/**
	 * Liefert den Wert der Wertpapier derzeit im Depot.
	 * 
	 * @return den Wert an Aktien im Depot
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	// double getValue() throws RemoteException;

	/**
	 * Zustzlich noch ein eMethode, um all historyValues dieser Wertpapier zu
	 * erhalten.
	 * 
	 * @return Liste aller historyValues dieser Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 DBIterator<?> getValueHistory() throws RemoteException;

	/**
	 * Setzt die Beschreibung der Aktien.
	 * 
	 * @param comment
	 *            die Beschreibung der Aktien.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setComment(String comment) throws RemoteException;

	/**
	 * Setzt den lastTradeDate der Wertpapier.
	 * 
	 * @param lastTradeDate
	 *            der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setLastTradeDate(Date lastTradeDate) throws RemoteException;

	/**
	 * Setzt den lastTradePriceOnly der Wertpapier.
	 * 
	 * @param lastTradePriceOnly
	 *            der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setLastTradePriceOnly(double lastTradePriceOnly) //No PDM // NOPMD by munstfr on 23.09.16 12:16
			throws RemoteException;

	/**
	 * Setzt den lastTradeTime der Wertpapier.
	 * 
	 * @param lastTradeTime
	 *            der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setLastTradeTime(Time lastTradeTime) throws RemoteException;

	/**
	 * Setzt den Name der Wertpapier.
	 * 
	 * @param name
	 *            Name der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setName(String name) throws RemoteException;

	/**
	 * Setzt den previousClose der Wertpapier.
	 * 
	 * @param previousClose
	 *            der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setPreviousClose(double previousClose) throws RemoteException;

	/**
	 * Setzt das Symbol der Wertpapier.
	 * 
	 * @param symbol
	 *            Symbol der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setSymbol(String symbol) throws RemoteException;

	/**
	 * Setzt den Wert der Aktien derzeit im Depot.
	 * 
	 * @param value
	 *            den Wert der Aktien im Depot
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	// public void setValue(double value) throws RemoteException;

	/**
	 * Liefert den yearLow Wert des Wertpapier.
	 * 
	 * @return yearLow der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 double getYearLow() throws RemoteException;

	/**
	 * Setzt den yearLow Wert der Wertpapiers.
	 * 
	 * @param yearLow
	 *            den yearLow Wert der Aktien.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setYearLow(double yearLow) throws RemoteException;

	/**
	 * Liefert den yearHigh Wert des Wertpapier.
	 * 
	 * @return yearHigh der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 double getYearHigh() throws RemoteException;

	/**
	 * Setzt den yearHigh Wert der Wertpapiers.
	 * 
	 * @param yearHigh
	 *            den yearHigh Wert der Aktien.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setYearHigh(double yearHigh) throws RemoteException;

	/**
	 * Liefert den change Wert des Wertpapier.
	 * 
	 * @return change der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 double getChange() throws RemoteException;

	/**
	 * Setzt den change Wert der Wertpapiers.
	 * 
	 * @param change
	 *            den change Wert der Aktien.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setChange(double change) throws RemoteException;

	/**
	 * Liefert den changeInPercentage Wert des Wertpapier.
	 * 
	 * @return changeInPercentage der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 double getChangeInPercentage() throws RemoteException;

	/**
	 * Setzt den changeInPercentage Wert der Wertpapiers.
	 * 
	 * @param changeInPercentage
	 *            den changeInPercentage Wert der Aktien.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	void setChangeInPercentage(double changeInPercentage) // NOPMD 
			throws RemoteException;

	/**
	 * Liefert den Type Wert des Wertpapier.
	 * 
	 * @return type der Wertpapier.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 String getType() throws RemoteException;

	/**
	 * Setzt den Type Wert der Wertpapiers.
	 * 
	 * @param type
	 *            den Type Wert der Wertpapiers.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setType(String type) throws RemoteException;

	/**
	 * Liefert die B&ouml;rse Wert des Wertpapiers.
	 * 
	 * @return exchange des Wertpapiers.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 String getExchange() throws RemoteException;

	/**
	 * Setzt die B&omul; des Wertpapiers.
	 * 
	 * @param exchange
	 *            die B&omul; des Wertpapiers.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setExchange(String exchange) throws RemoteException;

	/**
	 * Liefert die W&auml;hrung des Wertpapiers.
	 * 
	 * @return currency des Wertpapiers.
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 String getCurrency() throws RemoteException;

	/**
	 * Setzt die W&auml;hrung des Wertpapiers.
	 * 
	 * @param currency
	 *            die W&auml;hrung des Wertpapiers.
	 * 
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	 void setCurrency(String currency) throws RemoteException;

	/**
	 * Aktualisiert das Wertpapier mit den Angaben aus einen
	 * {@link de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierInfoBean}
	 * Objekts.
	 * 
	 * @param wpi
	 *            das WertpapierInfoBean Objekt.
	 * @throws RemoteException
	 */
	 void parseWertpapier(WertpapierInfoBean wpi) throws RemoteException;

	/**
	 * Liefert die Anzahl an Aktien im Depot, berechnet aus der OrderHistory
	 * 
	 * @return die Anzahl an Aktien im Depot
	 * @throws RemoteException
	 *             see {@link java.rmi.RemoteException}
	 */
	double getAnzahl() throws RemoteException;
	
	/**
	 * Liefert die Angabe, ob das Wertpapier in Yahoo gelistet ist oder nicht.
	 * Muss bei jeden erstellen neu berprft und gesetzt werden.
	 * 
	 * @return true falls der Wertpapier in Yahoo gelistet ist, false sonst
	 * @throws RemoteException  see {@link java.rmi.RemoteException}
	 */
	 boolean isInYahoo() throws RemoteException;
	
	/**
	 * Liefert die gesamten Ausgaben zum Erwerb der Wertpapier.
	 * 
	 * @return die gesamten Ausgaben zum Erwerb der Wertpapier.
	 * @throws RemoteException  see {@link java.rmi.RemoteException}
	 */
	
	 double getKauf() throws RemoteException;
	/**
	 * Liefert die gesamten zus&auml;tzlichen Kosten des Wertpapiers.
	 * 
	 * @return die zus&auml;tzlichen Kosten.
	 * @throws RemoteException  see {@link java.rmi.RemoteException}
	 */
	 double getAdditionalCosts() throws RemoteException;
	

	/**
	 * Liefert den Wert aller Papiere, die noch im Bestand sind.
	 * 
	 * @return den Gesamtwert (Anzahl * Kurs).
	 * @throws RemoteException see {@link java.rmi.RemoteException}
	 */
	 double getGesamtWert() throws RemoteException;

	/**
	 * Liefert das Gesamtergebnis der Papieres ber alle Kauf- und
	 * Verkaufsorder.
	 * 
	 * @return das Gesamtergebnis des Papieres
	 * @throws RemoteException see {@link java.rmi.RemoteException}
	 */
	 double getTotalGewinn() throws RemoteException;

	/**
	 * Liefert der realisierten Gewinn der Verkaufsorder.
	 * 
	 * @return den realisierten Gewinn
	 * @throws RemoteException see {@link java.rmi.RemoteException}
	 */
	 double getVerkauf() throws RemoteException;

}
