/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.finanzen.yahoo.finance;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.currencyexchangerate.ICurrencyExchangeRate;
import de.willuhn.logging.Logger;

/**
 * Implementierung des W&auml;hrungsumrechner mittels Yahoo Services.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class CurrencyConverterYahoo implements ICurrencyExchangeRate {

	/**
	 * Die interne Instanz
	 */
	private static CurrencyConverterYahoo instance;

	/**
	 * YAHOO W&auml;hrungssysmble f&uumL;r den US-Dollar.
	 */
	private final static String USD_SYSMBOL = "USD"; //$NON-NLS-1$

	/**
	 * Yahoo Standartantwort, falls nichts gefunden wurde.
	 */
	private final static String NOT_A_RESOURCE = "NaR"; //$NON-NLS-1$

	/**
	 * Privater Konstruktor um externe Instanziierung zu verhindern.
	 */
	private CurrencyConverterYahoo() {

	}

	/**
	 * Liefert die Instanz des Yahoo-W&auml;hrungsumrechner.
	 * 
	 * @return die Instanz des Yahoo-W&auml;hrungsumrechner.
	 */
	public static CurrencyConverterYahoo getInstance() {
		synchronized (CurrencyConverterYahoo.class) {
			if (CurrencyConverterYahoo.instance == null) {
				CurrencyConverterYahoo.instance = new CurrencyConverterYahoo();
			}
		}
		return CurrencyConverterYahoo.instance;
	}

	/**
	 * @see ICurrencyExchangeRate#exchangeRate(String, String)
	 */
	@Override
	public float exchangeRate(final String currencyFrom, final String currencyTo)
			throws YahooFinanceException {

		String request = ""; //$NON-NLS-1$
		URL url;
		try {
			request = "http://quote.yahoo.com/d/quotes.csv?s=" + currencyFrom //$NON-NLS-1$
					+ currencyTo + "=X&f=l1&e=.csv"; //$NON-NLS-1$

			url = new URL(request);
			final String responseString = YahooFinanceAPI.getInstance()
					.getResonseGETRequest(url, true);

			return Float.parseFloat(responseString);
		} catch (MalformedURLException e1) {
			throw (YahooFinanceException) new YahooFinanceException(Settings
					.getI18n().tr(
							"CurrencyConverterYahoo.malformedURL", request), //$NON-NLS-1$
					YahooFinanceExceptionType.OTHER).initCause(e1);
		}
	}

	/**
	 * @see ICurrencyExchangeRate#exchangeRate(String, String, Date)
	 */
	@Override
	public float exchangeRate(final String currencyFrom,
			final String currencyTo, final Date date)
			throws YahooFinanceException {

		final HistoricalExchanges histoExchanges = new HistoricalExchanges(date);
		return histoExchanges.getExchangeRate(currencyFrom, currencyTo);
	}

	/**
	 * Interne Klasse, die historische Umrechnungsraten zu einem Tag
	 * bereitstellt.
	 */
	private class HistoricalExchanges {

		/**
		 * Das Datum der Kurse
		 */
		private final transient Date date;

		/**
		 * Liste mit allen Umrechnugswerten W&auml;hrung <-> USD
		 */
		private transient List<CurrencyToUsdExchangeRate> exchangeHistoLst;

		/**
		 * Standartkonstruktor.
		 * 
		 * @param date
		 *            das Datum f&uuml; die Umrechnung.
		 * 
		 * @throws IllegalArgumentException
		 *             Das Datum darf nicht <i>NULL</i> sein.
		 */
		public HistoricalExchanges(final Date date)
				throws IllegalArgumentException {

			if (date == null) {
				throw new IllegalArgumentException(Settings.getI18n().tr(
						"CurrencyConverterYahoo.dateIsNull")); //$NON-NLS-1$
			}

			this.date = date;
			init();

		}

		/**
		 * Liefert die Umrechungsrate zum gegeben Datum.
		 * 
		 * @param fromCurrency
		 *            die Ausgangsw&auml;hrung.
		 * @param toCurrency
		 *            die Zielw&auml;hrung.
		 * @return die Umrechungsrate
		 * @throws YahooFinanceException
		 */
		private float getExchangeRate(final String fromCurrency,
				final String toCurrency) throws YahooFinanceException {

			float exchangerate;
			// Ist eine der beiden Whrungen USD
			if (USD_SYSMBOL.equals(fromCurrency)) {
				exchangerate = getUSDToCur(toCurrency);
			} else if ( USD_SYSMBOL.equals(toCurrency)) {
				exchangerate = 1 / getUSDToCur(fromCurrency);
			} else {
				final float fromToToUSD = 1 / getUSDToCur(fromCurrency);
				final float fromUSDtoTo = getUSDToCur(toCurrency);
				exchangerate = fromToToUSD * fromUSDtoTo;
			}

			return exchangerate;
		}

		/**
		 * Liefert Umtauschrate von USD nach W&aumlhrung.
		 * 
		 * @param currency
		 *            die Zielw&auhrung.
		 * @return Die Ummtauschrate.
		 * @throws YahooFinanceException
		 */
		protected float getUSDToCur(final String currency)
				throws YahooFinanceException {

			Float usdToCur = null;

			if (!USD_SYSMBOL.equals(currency)) {
				usdToCur = 1.0f;
			}
			final String symbol = currency + "=X"; //$NON-NLS-1$

			for (final CurrencyToUsdExchangeRate ehFrom : exchangeHistoLst) {
				if (ehFrom.getSymbol().equals(symbol)) {
					usdToCur = ehFrom.getPrice();
					break;
				}
			}

			if (usdToCur == null) {

				throw new YahooFinanceException(
						Settings.getI18n()
								.tr(Settings
										.getI18n()
										.tr("CurrencyConverterYahoo.currencyNotSupported", currency)), //$NON-NLS-1$
						YahooFinanceExceptionType.CURRENCY_NOT_SUPPORTED);
			}

			return usdToCur;

		}

		private void init() {

			URL url;
			final SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd", //$NON-NLS-1$
					Locale.GERMAN);
			final String request = "http://finance.yahoo.com/connection/currency-converter-cache?bypass=true&date=" //$NON-NLS-1$
					+ sdf.format(date);
			String responseJson = ""; //$NON-NLS-1$

			try {
				url = new URL(request);

				final String responseString = YahooFinanceAPI.getInstance()
						.getResonseGETRequest(url, false);

				if (!NOT_A_RESOURCE.equals(responseString)) {

					final Pattern pattern = Pattern
							.compile("(?s)YAHOO.Finance.CurrencyConverter.addConversionRates\\((.*)\\)"); //$NON-NLS-1$
					final Matcher matcher = pattern.matcher(responseString);
					if (matcher.find()) {
						responseJson = matcher.group(1);
					} else {
						return;
					}
				}

				final JSONObject responseObject = (JSONObject) JSONValue
						.parse(responseJson);
				final JSONObject list = (JSONObject) responseObject.get("list"); //$NON-NLS-1$
				final JSONArray resources = (JSONArray) list.get("resources"); //$NON-NLS-1$

				final DateFormat dfDateOnly = new SimpleDateFormat(
						"yyyy-MM-dd", Locale.GERMAN); //$NON-NLS-1$
				final DateFormat dfUtcTime = new SimpleDateFormat(
						"yyyy-MM-dd'T'HH:mm:ssZ", Locale.GERMAN); //$NON-NLS-1$
				// 2015-08-05T06:22:06+0000

				exchangeHistoLst = new ArrayList<CurrencyToUsdExchangeRate>();

				for (final Object exchangeHistory : resources) {
					final JSONObject exchHistoRes = (JSONObject) ((JSONObject) exchangeHistory)
							.get("resource"); //$NON-NLS-1$
					final CurrencyToUsdExchangeRate exchangeRate = new CurrencyToUsdExchangeRate(); // NOPMD

					exchangeRate.setClassName((String) exchHistoRes
							.get("classname")); //$NON-NLS-1$
					final JSONObject exchHistoResFlds = (JSONObject) exchHistoRes
							.get("fields"); //$NON-NLS-1$
					try {
						String date = (String) exchHistoResFlds.get("date"); //$NON-NLS-1$
						if (date == null) {
							date = (String) exchHistoResFlds.get("utctime"); //$NON-NLS-1$
							exchangeRate.setDate(dfUtcTime.parse(date));
						} else {
							exchangeRate.setDate(dfDateOnly.parse(date));
						}
					} catch (ParseException e) {
						Logger.warn(Settings
								.getI18n()
								.tr("CurrencyConverterYahoo.parseError", e.getMessage())); //$NON-NLS-1$
					}
					exchangeRate
							.setPrice(Float
									.parseFloat((String) exchHistoResFlds
											.get("price"))); //$NON-NLS-1$
					exchangeRate.setSymbol((String) exchHistoResFlds
							.get("symbol")); //$NON-NLS-1$
					exchangeRate.setType((String) exchHistoResFlds.get("type")); //$NON-NLS-1$

					exchangeHistoLst.add(exchangeRate);
				}

			} catch (MalformedURLException e) {
				Logger.warn(Settings.getI18n().tr(
						"CurrencyConverterYahoo.malformedURL" //$NON-NLS-1$
						, e.getLocalizedMessage()));
			} catch (YahooFinanceException e) {
				Logger.warn(Settings.getI18n().tr(
						"CurrencyConverterYahoo.generalError" //$NON-NLS-1$
						, e.getLocalizedMessage()));
			}

		}
	}

}
