/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.finanzen.yahoo.finance;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.time.DateUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.willuhn.logging.Logger;

/**
 * Yahoo Wertpapierhistory Bean
 * 
 * @author munstfr
 * @version 1.0
 */
public class WertpapierHistory {

	/**
	 * DatenAbfrage bei Yahoo erfolgreich
	 */
	private final transient boolean dataRequestOK;

	/**
	 * Anfangsdatum der Werte
	 */
	private final Date fromDate;

	/**
	 * Yahoo-Symbol
	 */
	private final String symbol;

	/**
	 * Enddatum der Werte
	 */
	private final Date toDate;

	/**
	 * Liefert eine Liste mit den historischen Werten des Wertpapiers
	 */
	private final transient List<HistoryValueBean> wphv = new ArrayList<HistoryValueBean>();

	/**
	 * Konstruktor mit Angabe des Wertpapiers.
	 * 
	 * @param symbol
	 *            Das YAhoo-Symbol des Wertpapiers.
	 * @throws YahooFinanceException
	 */
	public WertpapierHistory(final String symbol) throws YahooFinanceException {

		YahooFinanceException yahooFinException = null;
		this.symbol = symbol;

		// Erst die Daten laden
		try {
			loadData();
		} catch (YahooFinanceException ye) {
			yahooFinException = ye;
		}
		if (yahooFinException == null) {
			this.dataRequestOK = true;
		} else {
			this.dataRequestOK = false;
			throw yahooFinException;
		}

		// Dann sortieren.
		Collections.sort(this.wphv);

		// und das Anfangs- und Enddatum setzten
		this.fromDate = this.wphv.get(0).getDate();
		this.toDate = this.wphv.get(this.wphv.size() - 1).getDate();

	}

	/**
	 * Liefert das Anfangsdatum
	 * 
	 * @return fromDate
	 */
	public Date getFromDate() {
		return fromDate;
	}

	/**
	 * Liefert eine Liste mit den historischen Werten des Wertpapiers.
	 * 
	 * @return wphv Liste mit den HistoryValueBean
	 */
	public List<HistoryValueBean> getHistoryValues() {
		return wphv;
	}

	/**
	 * Liefert das Yahoo-Symbol
	 * 
	 * @return symbol
	 */
	public String getSymbol() {
		return symbol;
	}

	/**
	 * Liefert die historischen Werte f&uuml;r eine angegeben Zeitraum.
	 * 
	 * @param fromDate
	 *            das Anfangsdatum
	 * @param toDate
	 *            das Enddatum
	 * @return Die Liste mit HistoryValueBean
	 */
	public List<HistoryValueBean> getTimeRangeValues(final Date fromDate,
			final Date toDate) {

		final List<HistoryValueBean> rangeValues = new ArrayList<HistoryValueBean>();

		for (final HistoryValueBean value : wphv) {
			final long dateInt = value.getDate().getTime();
			if (dateInt > fromDate.getTime() && dateInt < toDate.getTime()) {
				rangeValues.add(value);
			}
		}

		return rangeValues;
	}

	/**
	 * Liefert die historischen Werte f&uuml;r ein angegebenes Datum. Falls kein
	 * Daten f&uuml; das angegeben Datum zur Verf&uuml;gung stehen, wird der
	 * n&auml;chste davorliegende Handelstag genommen. Liegt kein Tage vor
	 * DATUM, werden die Daten des ersten zur Verf&uuml;gung stehenden Tages
	 * verwendet. Liegen gar keine Daten vor wir NULL zur&uuml;ckgegeben.
	 * 
	 * @param date
	 *            das Datum
	 * @return Der HistoryValueBean
	 */
	public HistoryValueBean getValuesAtDate(final Date date) {

		HistoryValueBean returnValue = null;

		if (!wphv.isEmpty()) {
			returnValue = wphv.get(0);

			for (final HistoryValueBean value : wphv) {

				final Date dateValue = value.getDate();
				if (DateUtils.isSameDay(date, dateValue)) {
					// Gesuchtes Datum gefunden
					returnValue = value;
					break;
				} else if (date.before(dateValue)) {
					// Wir sind am gesuchten Datum vorbei
					break;
				}
				returnValue = value;
			}
		}
		return returnValue;
	}

	/**
	 * Liefert das Enddatum
	 * 
	 * @return toDate
	 */
	public Date getToDate() {
		return toDate;
	}

	/**
	 * War der Datenabruf OK.
	 * 
	 * @return the dataRequestOK
	 */
	public Boolean isDataRequestOK() {
		return dataRequestOK;
	}

	/**
	 * @throws YahooFinanceException
	 * 
	 */
	private void loadData() throws YahooFinanceException {

		wphv.clear();
		String request = ""; //$NON-NLS-1$
		URL url;
		try {
			request = "http://ichart.yahoo.com/table.csv?s=" //$NON-NLS-1$
					+ URLEncoder.encode(symbol, "UTF-8"); //$NON-NLS-1$
			url = new URL(request);
			Logger.debug(request);
		} catch (MalformedURLException e1) {
			final String message = Settings.getI18n().tr(
					"WertpapierHistory.malformedURL", request); //$NON-NLS-1$
			throw (YahooFinanceException) new YahooFinanceException(message,
					YahooFinanceExceptionType.OTHER).initCause(e1);
		} catch (UnsupportedEncodingException e) {
			throw (YahooFinanceException) new YahooFinanceException(Settings
					.getI18n().tr("WertpapierHistory.encodingNotSupported"), //$NON-NLS-1$
					YahooFinanceExceptionType.OTHER).initCause(e);
		}

		try {
			final String responseString = YahooFinanceAPI.getInstance()
					.getResonseGETRequest(url, true);
			final BufferedReader bufReader = new BufferedReader(
					new StringReader(responseString));
			final DateFormat df1 = new SimpleDateFormat("yyyy-MM-dd", //$NON-NLS-1$
					Locale.GERMAN);
			final GregorianCalendar gregCalendar = new GregorianCalendar();

			// Lese Kopfzeile
			bufReader.readLine();
			// Und nun den Rest
			while (true) {
				final String line = bufReader.readLine();
				if (line == null) {
					break;
				}

				final String[] values = line.split(","); //$NON-NLS-1$

				final HistoryValueBean wphv = new HistoryValueBean(); // NOPMD
				wphv.setSymbol(symbol);
				try {
					final Date date = df1.parse(values[0]);
					gregCalendar.setTime(date);
					gregCalendar.set(Calendar.HOUR_OF_DAY, 2);
					gregCalendar.set(Calendar.MINUTE, 0);
					gregCalendar.set(Calendar.SECOND, 0);
					wphv.setDate(gregCalendar.getTime());
				} catch (ParseException e) {
					Logger.warn(Settings.getI18n().tr(
							"WertpapierHistory.parseDateError" //$NON-NLS-1$
							, e.getMessage()));
				}
				wphv.setOpen(Double.parseDouble(values[1]));
				wphv.setHigh(Double.parseDouble(values[2]));
				wphv.setLow(Double.parseDouble(values[3]));
				wphv.setClose(Double.parseDouble(values[4]));
				wphv.setVolume(Double.parseDouble(values[5]));
				wphv.setAdjClose(Double.parseDouble(values[6]));

				this.wphv.add(wphv);

			}
		} catch (IOException e) {
			throw (YahooFinanceException) new YahooFinanceException(Settings
					.getI18n().tr("WertpapierHistory.readingGeneralError"), //$NON-NLS-1$
					YahooFinanceExceptionType.OTHER).initCause(e);
		}

	}

}
