/*
 * Copyright 2016 frankmuenster.de
 * 
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * 
 * http://ec.europa.eu/idabc/eupl
 * 
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.input;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.AbstractInput;
import de.willuhn.jameica.gui.input.LabelInput;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.system.OperationCanceledException;

/**
 * Das ist eine Erweiterung des {@link LabelInput} Objekts. Es kann nun ein
 * Tooltip sowie einen Rand haben. Durch setzten von tooltip auf
 * <code>null</code> kann der tooltip gel&ouml;scht werden.
 * 
 * @author munstfr
 */
public class LabelInputExtended extends AbstractInput {

	/**
	 * Das AusgabeLabel
	 */
	private transient Label label;

	/**
	 * Der Labeltext
	 */
	private transient String value;

	/**
	 * Die Textfarbe
	 */
	private transient Color color;

	/**
	 * Der Tooltiptext
	 */
	private transient String tooltip;

	/**
	 * Zeige Rahmen
	 */
	private transient Boolean border;

	/**
	 * Erzeugt ein neues Label mit dem angegebenen Wert.
	 * 
	 * @param value
	 *            anzuzeigender Wert.
	 */
	public LabelInputExtended(final String value) {
		this(value, "", false); //$NON-NLS-1$
	}

	/**
	 * Erzeugt ein Label mit Wert und Tooltip.
	 * 
	 * @param value
	 *            anzuzeigender Wert.
	 * @param tooltip
	 *            anzuzeigender Tooltip
	 */
	public LabelInputExtended(final String value, final String tooltip) {
		this(value, tooltip, false);
	}

	/**
	 * Erzeugt ein Label mit Wert, ohne Tooltip und Randfalls
	 * <code>border=true</code>.
	 * 
	 * @param value
	 *            anzuzeigender Text
	 * @param border
	 *            mit Rand falls <code>true</code>, sonst ohne
	 */
	public LabelInputExtended(final String value, final Boolean border) {
		this(value, "", border); //$NON-NLS-1$
	}

	/**
	 * Erzeugt ein Label mit Wert, Tooltip und Rand, falls
	 * <code>border=true</code>.
	 * 
	 * @param value
	 *            anzuzeigender Wert.
	 * @param tooltip
	 *            anzuzeigender Tooltip
	 * @param border
	 *            mit Rand falls <code>true</code>, sonst ohne
	 */
	public LabelInputExtended(final String value, final String tooltip,
			final Boolean border) {
		super();
		this.value = value;
		this.tooltip = tooltip;
		if (border == null) {
			this.border = false;
		} else {
			this.border = border;
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#getControl()
	 */
	@Override
	public Control getControl() {
		if (border) {
			label = GUI.getStyleFactory().createLabel(getParent(), SWT.BORDER);
		} else {
			label = GUI.getStyleFactory().createLabel(getParent(), SWT.NONE);
		}
		label.setText(value == null ? "" : SWTUtil.escapeLabel(value)); //$NON-NLS-1$
		label.setToolTipText(tooltip == null ? null : SWTUtil
				.escapeLabel(this.tooltip));

		// label.setToolTipText(null) :
		// label.setToolTipText(SWTUtil.escapeLabel(this.tooltip));

		if (this.color != null) {
			this.label.setForeground(this.color.getSWTColor());
		}
		return label;

	}

	/**
	 * Liefert den angezeigten Text.
	 * 
	 * @see de.willuhn.jameica.gui.input.Input#getValue()
	 */
	@Override
	public Object getValue() {
		return value;
	}

	/**
	 * Liefert den Tooltip Text, oder null, falls kein Tooltip vorhanden ist.
	 * 
	 * @return den Tooltip Text.
	 */
	public String getToolTip() {
		return tooltip;
	}

	/**
	 * Liefert die Randeinstellung.
	 * 
	 * @return <code>true</code> falls das label einen Rand hat, sonst
	 *         <code>false</code>.
	 */
	public Boolean isBorder() {
		return border;
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#focus()
	 */
	@Override
	public void focus() {
		// hier nix zu tun
	}

	/**
	 * Erwartet ein Objekt des Typs <code>java.lang.String</code>.
	 * 
	 * @see de.willuhn.jameica.gui.input.Input#setValue(java.lang.Object)
	 */
	@Override
	public void setValue(final Object value) {
		if (value != null) {
			this.value = (String) value;
			if (this.label != null && !this.label.isDisposed()) {
				this.label.setText(SWTUtil.escapeLabel(value.toString()));
				this.label.redraw();
			}
		}
	}

	/**
	 * Legt die Text-Farbe fest.
	 * 
	 * @param color
	 *            Text-Farbe.
	 */
	public void setColor(final Color color) {
		this.color = color;
		if (this.label != null && !this.label.isDisposed()) {
			this.label.setForeground(this.color.getSWTColor());
		}
	}

	/**
	 * @param tooltip
	 *            Der Tooltiptext
	 */
	public void setToolTip(final String tooltip) {

		this.tooltip = tooltip;
		if (this.label != null && !this.label.isDisposed()) {
			label.setToolTipText(this.tooltip == null ? null : SWTUtil
					.escapeLabel(this.tooltip));
			this.label.redraw();
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#disable()
	 */
	@Override
	public void disable() {
		// hier nix zu tun
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#enable()
	 */
	@Override
	public void enable() {
		// hier nix zu tun
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#isEnabled()
	 */
	@Override
	public boolean isEnabled() {
		return false;
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(final boolean enabled) {
		// Nee, das machen wir nicht
	}

	/**
	 * @see de.willuhn.jameica.gui.input.AbstractInput#update()
	 */
	@Override
	protected void update() throws OperationCanceledException {
		// Wir machen hier nichts
	}

}
